% This program replicates tables 8, 9, and 10 in "A Static Capital Buffer is 
% Hard to Beat" by Matthew Canzoneri, Behzad Diba, Luca Guerrieri 
% and Arsenii Mishin.
%
% This program considers the change in the distance function relative to
% the results in call_tables_2_3_and_4.m from turning off volatility
% shocks.


clear all

setpath_windows

global M_ oo_
global changes_names changes_vec


overwrite_results = 0;  % set to 1 to overwrite the results used for the paper
load_from_disk = 1;     % set to 1 to load initial guess from disk

modnam = 'model_calibration';

% import observed data
data_table = readtable('calibration_dataset.xlsx');

%dow_return = log(data_table.dow(2:end))-log(data_table.dow(1:end-1));
%dow_return_detrended = 100*bpass(dow_return,6,32);

% detrend data
gdp_detrended = 100*bpass(log(data_table.gdp),6,32);

investment_detrended = 100*bpass(log(data_table.investment),6,32);
investment_price_detrended = 100*bpass(log(data_table.investment_price)-log(data_table.consumption_price),6,32);

% assemble series for moment matching
dataset = [gdp_detrended(2:end) investment_detrended(2:end) investment_price_detrended(2:end)];
var_labels =char('GDP','Investment','Investment Price');


% select strategy for creating the GMM weighting matrix
%optwt =1; % weighting matrix is diagonal optimal SMM matrix
optwt =2; % optimal GMM matrix
%optwt = -1; % weighting matrix is the identity matrix

% create the GMM weighting matrix
[data_momm,wtmat] = wtm_rm_noskew_opt(dataset,optwt);


% solve the model
eval(['dynare ',modnam,' noclearall'])


if load_from_disk
    % load starting guess from disk
    load my_estimates;
    init_theta = distance_param;
else
   
   %specify starting guess (only necessary the first time the codes are
   %run)
        init_theta = [
   0.009339783076469
   0.005182948520449
   0.787857734398795
   0.952526747477005
   0.058850743081795
   0.928089965122636
            ];
end


% parameters to be calibrated
changes_names = char('sigma_eps_a','sigma_eps_isp',...
    'rhop_a','rho_isp','phip_i','kappap');

% test the distance function
[dist model_momm] = distance_function(init_theta,data_momm,wtmat);

% options for the numerical minimizer
options = optimset('display','iter','MaxIter',200,'MaxFunEvals',1e10,'TolFun',10e-3,'TolX',1e-4);

% iterate through guesses -- set a number for the rep loop; fminsearch has
% better convergence properties if we set a low maxfunevals (above) and
% call it many times --- we have reset the loop to 1 iteration after
% ascertaining convergence
thetaunc = init_theta;
if overwrite_results
    for rep = 1:1
        rep
        [thetaunc,fval,exitflag] = ...
            fminsearch('distance_function',init_theta,options,data_momm,wtmat);
        init_theta = thetaunc;
        distance_param = thetaunc;
        save my_estimates distance_param
    end
end


[dist model_momm] = distance_function(init_theta,data_momm,wtmat);

[model_momm data_momm]
%%
nvars = size(var_labels,1);
this_mom = 0;
moment_labels = char('')
for i = 1:nvars
    for j=i:nvars
        this_mom = this_mom+1;
        if i == j
            moment_labels = strvcat(moment_labels,['Var(',deblank(var_labels(i,:)),')']);
            display(['Var(',deblank(var_labels(i,:)),') in the model is ',num2str(model_momm(this_mom)),' and in the data is ',num2str(data_momm(this_mom))])
        else
            moment_labels = strvcat(moment_labels,['Corr(',deblank(var_labels(i,:)),',', deblank(var_labels(j,:)),')']);
            display(['The covariance of ',deblank(var_labels(i,:)),' with ',deblank(var_labels(j,:)),' in the model is ',num2str(model_momm(this_mom)),' and in the data is ',num2str(data_momm(this_mom))])
            
        end
    end
end

for i = 1:nvars
    this_mom = this_mom+1;
    moment_labels = strvcat(moment_labels,['Autocorr(',deblank(var_labels(i,:)),')']);
    display(['The auto-covariance of ',deblank(var_labels(i,:)),' is ',num2str(model_momm(this_mom)),' and in the data is ',num2str(data_momm(this_mom))])
    
end
%%

changes_vec = thetaunc;


nperiods = 2000;
irf_shock = char('eps_a','eps_isp');

randn('seed',1);
nshocks = size(irf_shock,1);
shockssequence = randn(nperiods,nshocks);

[zdatalinear, zdatass, oobase_, Mbase_ ] = ...
    solve_no_constraint_no_parse(shockssequence,irf_shock,nperiods);

ytot_pos = strmatch('ytot',Mbase_.endo_names,'exact');
ltot2ytot_pos = strmatch('ltot2ytot',Mbase_.endo_names,'exact');
gamma_var_pos = strmatch('gamma_var',Mbase_.endo_names,'exact');
ginvest_pos = strmatch('ginvest',Mbase_.endo_names,'exact');
q_pos = strmatch('q',Mbase_.endo_names,'exact');
r_e_pos = strmatch('r_e',Mbase_.endo_names,'exact');

f0_ytot_data = zdatalinear(:,ytot_pos);
f0_ytot_ss = zdatass(ytot_pos);
f0_ltot2ytot_data = zdatalinear(:,ltot2ytot_pos);
f0_ginvest_data = zdatalinear(:,ginvest_pos);
f0_ginvest_ss = zdatass(ginvest_pos);
f0_gamma_var_data = zdatalinear(:,gamma_var_pos);
f0_q_data = zdatalinear(:,q_pos);
f0_r_e_data = zdatalinear(:,r_e_pos);


gdp = 100*bpass(f0_ytot_data/f0_ytot_ss,6,32);
ltot2ytot = 100*bpass(f0_ltot2ytot_data,6,32);
gamma_var = 100*bpass(f0_gamma_var_data,6,32);
ginvest = 100*bpass(f0_ginvest_data/f0_ginvest_ss,6,32);
q = 100*bpass(f0_q_data,6,32);
r_e =100*bpass(f0_r_e_data,6,32);



tot_gdp_var = var(gdp);
tot_ltot2ytot_var = var(ltot2ytot);
tot_gamma_var = var(gamma_var);
tot_ginvest_var = var(ginvest);
tot_q_var = var(q);
tot_r_e_var = var(r_e);

% repeat the variance calculation only turning on the TFP shock
nperiods = 2000;
irf_shock = char('eps_a');


[zdatalinear, zdatass, oobase_, Mbase_ ] = ...
    solve_no_constraint_no_parse(shockssequence(:,1),irf_shock,nperiods);

ytot_pos = strmatch('ytot',Mbase_.endo_names,'exact');
gamma_var_pos = strmatch('gamma_var',Mbase_.endo_names,'exact');


f0_ytot_data = zdatalinear(:,ytot_pos);
f0_ytot_ss = zdatass(ytot_pos);
f0_ltot2ytot_data = zdatalinear(:,ltot2ytot_pos);
f0_ginvest_data = zdatalinear(:,ginvest_pos);
f0_ginvest_ss = zdatass(ginvest_pos);
f0_gamma_var_data = zdatalinear(:,gamma_var_pos);
f0_q_data = zdatalinear(:,q_pos);
f0_r_e_data = zdatalinear(:,r_e_pos);


gdp = 100*bpass(f0_ytot_data/f0_ytot_ss,6,32);
ltot2ytot = 100*bpass(f0_ltot2ytot_data,6,32);
gamma_var = 100*bpass(f0_gamma_var_data,6,32);
ginvest = 100*bpass(f0_ginvest_data/f0_ginvest_ss,6,32);
q = 100*bpass(f0_q_data,6,32);
r_e =100*bpass(f0_r_e_data,6,32);

part_gdp_var_a = var(gdp);
part_gamma_var_a = var(gamma_var);
part_ltot2ytot_var_a = var(ltot2ytot);
part_ginvest_var_a = var(ginvest);
part_q_var_a = var(q);
part_r_e_var_a = var(r_e);




% get simulated data with only the ISP shock turned on
nperiods = 2000;
irf_shock = char('eps_isp');

[zdatalinear, zdatass, oobase_, Mbase_ ] = ...
    solve_no_constraint_no_parse(shockssequence(:,2),irf_shock,nperiods);

ytot_pos = strmatch('ytot',Mbase_.endo_names,'exact');
gamma_var_pos = strmatch('gamma_var',Mbase_.endo_names,'exact');


f0_ytot_data = zdatalinear(:,ytot_pos);
f0_ytot_ss = zdatass(ytot_pos);
f0_ltot2ytot_data = zdatalinear(:,ltot2ytot_pos);
f0_ginvest_data = zdatalinear(:,ginvest_pos);
f0_ginvest_ss = zdatass(ginvest_pos);
f0_gamma_var_data = zdatalinear(:,gamma_var_pos);
f0_q_data = zdatalinear(:,q_pos);
f0_r_e_data = zdatalinear(:,r_e_pos);


gdp = 100*bpass(f0_ytot_data/f0_ytot_ss,6,32);
ltot2ytot = 100*bpass(f0_ltot2ytot_data,6,32);
gamma_var = 100*bpass(f0_gamma_var_data,6,32);
ginvest = 100*bpass(f0_ginvest_data/f0_ginvest_ss,6,32);
q = 100*bpass(f0_q_data,6,32);
r_e =100*bpass(f0_r_e_data,6,32);


part_gdp_var_isp = var(gdp);
part_ltot2ytot_var_isp = var(ltot2ytot);
part_gamma_var_isp = var(gamma_var);
part_ginvest_var_isp = var(ginvest);
part_q_var_isp = var(q);
part_r_e_var_isp = var(r_e);


%%

gdp_var_share = [round(100*part_gdp_var_a/tot_gdp_var)
    100-round(100*part_gdp_var_a/tot_gdp_var)
    ];

ltot2ytot_var_share = [round(100*part_ltot2ytot_var_a/tot_ltot2ytot_var)
    100-round(100*part_ltot2ytot_var_a/tot_ltot2ytot_var)
    ];

gamma_var_share =[round(100*part_gamma_var_a/tot_gamma_var)
    100-round(100*part_gamma_var_a/tot_gamma_var)
    ];

ginvest_var_share =[round(100*part_ginvest_var_a/tot_ginvest_var)
    100-round(100*part_ginvest_var_a/tot_ginvest_var)
    ];

q_var_share =[round(100*part_q_var_a/tot_q_var)
    100-round(100*part_q_var_a/tot_q_var)
    ];

%%
sideways_switch = 0;
caption = '';

texfilename = 'table_moment';
columnlabels = char('Data','Model');
rowlabels = moment_labels;
table_mat = [data_momm, model_momm];
column_precision = [2, 2];
maketable(table_mat,texfilename,columnlabels,rowlabels,sideways_switch,caption,column_precision)


texfilename = 'table_calibration';
columnlabels = char('AR(1) param.','Innov. St. Dev.');
rowlabels = char('TFP','ISP');
table_mat = [thetaunc([3,4]), thetaunc([1,2])];
column_precision = [2, 4];
maketable(table_mat,texfilename,columnlabels,rowlabels,sideways_switch,caption,column_precision)

texfilename = 'table_calibration_share';
columnlabels = char('var(GDP)','var(invest.)','var(invest. p.)','var(gamma)','var(credit/GDP)');
rowlabels = char('TFP','ISP');
table_mat_share =[gdp_var_share, ginvest_var_share, q_var_share, gamma_var_share, ltot2ytot_var_share];
column_precision = [0 0 0 0 0];
maketable(table_mat_share,texfilename,columnlabels,rowlabels,sideways_switch,caption,column_precision)